#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU Lesser General Public License    #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU Lesser General Public License for more details.                  #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

require_relative '../helper'

class ExecuteShellTest < Test::Unit::TestCase
  Command = {
    :ls => {
      :linux => 'ls',
      :mingw => 'dir',
      :macosx => 'ls',
    }
  }

  def setup
    @obj = @class.new
    @working_path = Dir.getwd
    @home_path = File.expand_path(File.join('~', '..'))
    @user = File.basename(File.expand_path('~'))
  end

  def test_supported_systems
    assert_equal [:linux, :mingw, :macosx], @class.supported_systems
  end

  def test_supported_flag
    assert_equal true, @class.supported?
    assert_equal true, @class.supported?(:linux)
    assert_equal true, @class.supported?('linux')
    assert_equal true, @class.supported?(:mingw)
    assert_equal true, @class.supported?('mingw')
    assert_equal false, @class.supported?(:my_os)
    assert_equal false, @class.supported?('my_os')
  end

  def test_raise_not_implemented_error
    assert_raise(NotImplementedError) { @class.raise_not_implemented('method') }
  end

  def test_basic_run
    shell_result = @class.run("echo 'hello world'")
    assert_equal ShellResult, shell_result.class
    check_all 'hello world', '', true, shell_result
  end

  def test_bad_command
    result = @class.run('my_bad_command')

    case Platform::IMPL
      when :linux
        check_all '',
          'bash: line 1: my_bad_command: command not found',
          false, result
      when :mingw
        ExecuteShell.raise_not_implemented __method__
    end
  end

  def test_change_directory_parameter
    command = get_command(:ls)

    result = @class.run(command, @home_path)

    assert File.expand_path(Dir.getwd) != @home_path

    assert_equal true, result.success?
    assert_equal '', result.err

    assert result.out.index(@user),
      "'#{command}' does not appear to include " +
      "the current user's folder (#{@user}), " +
      "which implies that the path was not changed prior to execution."
  end

  def test_run_success
    assert_equal true, @class.run_success?("echo 'hello world'")
    assert_equal false, @class.run_success?('my_bad_command')
  end

  def test_run_out
    assert_equal 'hello world', @class.run_out("echo 'hello world'")
    assert_equal '', @class.run_out('my_bad_command')
  end

  def test_run_to_s
    assert_equal 'hello world', @class.run_to_s("echo 'hello world'")
    assert_equal 'bash: line 1: my_bad_command: command not found',
      @class.run_to_s('my_bad_command')
  end

  def test_run_success_err
    assert_equal [true, ''], @class.run_success_err("echo 'hello world'")
    assert_equal [false, 'bash: line 1: my_bad_command: command not found'],
      @class.run_success_err('my_bad_command')
  end

  def test_run_success_err_out
    assert_equal [true, '', 'hello world'],
      @class.run_success_err_out("echo 'hello world'")
    assert_equal [false, 'bash: line 1: my_bad_command: command not found', ''],
      @class.run_success_err_out('my_bad_command')
  end

  def test_respond_to
    assert_respond_to @class, :run_success?
    assert_respond_to @class, :run_success
    assert_respond_to @class, :run_success_err_out
    assert_respond_to @class, :run_err_out_success
    assert_respond_to @class, :run_success_to_s
    assert_equal false, @class.respond_to?(:run_success_blah)
  end

  ############################################################################
  private
  ############################################################################

  def get_command(cmd)
    return Command[cmd][Platform::IMPL]
  end

  def check_all(out, err, success, result)
    assert_equal out, result.out
    assert_equal err, result.err
    assert_equal success, result.success?
  end
end
